#!/usr/bin/env python
from distutils.sysconfig import get_python_lib
import inspect
import re
import os
import sys
from isstring import isstring
from find_module import find_module
from public import public
from stdin import STDIN


def getname(path):
    matches = re.findall('([\w+]*)', os.path.basename(path).split(".")[0])
    if matches:
        return matches[0].replace("-", "_")


def egg_distname(path):
    if path.find(".egg/") > 0 or os.path.splitext(path)[1] == ".egg":
        for name in path.split("/"):
            if os.path.splitext(name)[1] == ".egg":
                matches = re.findall('([\w+]*)', name)
                if matches:
                    return matches[0].replace("-", "_")


def sp_distname(path):
    if path.find("/site-packages/") > 0:
        sp = path.split("site-packages")[0] + "site-packages"
        pkgname = path.split("site-packages")[1].split("/")[1]
        listdir = os.listdir(sp)
        for basename in listdir:
            folder = os.path.join(sp, basename)
            txt = os.path.join(folder, "top_level.txt")
            if not (os.path.exists(txt) and os.path.isfile(txt)):
                continue
            top_level = open(txt).read()
            if top_level.rstrip() != pkgname:
                continue
            matches = re.findall('([\w+]*)', basename)
            if not matches:
                continue
            _distname = matches[0].replace("-", "_")
            if pkgname.lower() == _distname.lower() and pkgname != _distname:
                return pkgname
            return _distname
        # site-packages/modname.py
        if os.path.splitext(pkgname)[1] in [".py", ".pyc"]:
            return os.path.splitext(pkgname)[0]
        # site-packages/pkgname/
        return pkgname


def egg_info_distname(path):
    # distname-x.y.z-py2.7.egg-info
    # distname.egg-link
    if path.find(".egg-info") > 0 or path.find(".egg-link") > 0:
        matches = re.findall('([\w+]*)', path.split(".")[0])
        if matches:
            return matches[0].replace("-", "_")


def egg_link_distname(path):
    # distname-x.y.z-py2.7.egg-info
    # distname.egg-link
    if path.find(".egg-info") > 0 or path.find(".egg-link") > 0:
        matches = re.findall('([\w+]*)', path.split(".")[0])
        if matches:
            return matches[0].replace("-", "_")


def path_distname(path):
    # .egg
    if path.find(".egg/") > 0 or os.path.splitext(path)[1] == ".egg":
        return egg_distname(path)
    if path.find("/site-packages/") > 0:
        return sp_distname(path)
    else:  # develop?
        stdlib = get_python_lib(standard_lib=False)
        listdir = os.listdir(stdlib)
        for basename in listdir:
            filename, ext = os.path.splitext(basename)
            if ext == ".egg-link":  # distname.egg-link
                file = os.path.join(stdlib, basename)
                develop_path = open(file).read().splitlines()[0]
                if path.find(develop_path) == 0:
                    return filename.replace("-", "_")
    # unknown
    return getname(path)


def module_distname(module):
    module = inspect.getmodule(module)
    if hasattr(module, "__file__"):  # file.py
        path = module.__file__
        return distname(path)
    else:  # in memory module
        return module.__name__


@public
def distname(obj):
    """return distribution name"""
    if isstring(obj):  # text
        if "/" in obj:  # path
            return path_distname(obj)
        # distname-x.y.z-py2.7.egg-info
        # distname.egg-link
        if obj.find(".egg-info") > 0 or obj.find(".egg-link") > 0:
            matches = re.findall('([\w+]*)', obj.split(".")[0])
            if matches:
                return matches[0].replace("-", "_")
        # pkgname?
    if inspect.ismodule(obj):
        return module_distname(obj)
    module = inspect.getmodule(obj)
    if module:  # module
        return module_distname(module)
    find = find_module(obj)
    if find:
        return path_distname(find)
    name = str(obj)
    name = name.lstrip().rstrip()
    matches = re.findall('([\w+]*)', os.path.basename(name))
    if matches:
        return matches[0]

if __name__ == "__main__":
    # 1) python -m distname "pkgname"
    # 2) echo "pkgname" | python -m distname
    # print(distname("future"))
    if len(sys.argv) == 2:
        sys.exit(0)
    if STDIN:  # echo "pkgname" | python -m distname
        for l in STDIN.splitlines():
            if l:
                name = distname(l)
                if name:
                    print(name)
        sys.exit(0)
